@@ -319,24 +319,26 @@ class BaseIndexedModel(BaseModel, metaclass=BaseIndexedModelMetaclass):
             )
 
     @classmethod
-    def index_table_rebuild(cls, index_names=None):
+    def index_table_rebuild(cls, *args, callback=None):
         """
         Resets a model indexation tables.
 
         This method should be called if an attribute became indexed in an already
         populated database.
 
-        :param index_names: A list of index names to reset. If `None`, all the
-                            indexes will be reseted. The primary index cannot be
-                            resetted.
+        :param *args: A list of index names to reset. If `None`, all the
+                      indexes will be reseted. The primary index cannot be
+                      resetted.
+        :param callback: A callback that is called each item a model instance is
+                         iterated.
         """
-        if not index_names:
+        if not args:
             indexes = cls.indexes.values()
         else:
             indexes = [
                 index
                 for index_name, index in cls.indexes.items()
-                if index_name in index_names
+                if index_name in args
             ]
 
         for index in indexes:
@@ -345,11 +347,14 @@ class BaseIndexedModel(BaseModel, metaclass=BaseIndexedModelMetaclass):
 
             index.delete()
 
-        for m in cls.all():
+        for i, m in enumerate(cls.all()):
             for index in indexes:
                 if not index.details.primary:
                     index.add_item(m)
 
+            if callback and callback(i, m) is False:
+                break
+
     @classmethod
     def filter(cls, predicate=None, **kwargs):
         """Shortcut for :func:`sheraf.queryset.QuerySet.filter`.
@@ -526,7 +531,7 @@ class BaseIndexedModel(BaseModel, metaclass=BaseIndexedModelMetaclass):
             if not self._is_indexable(index):
                 warnings.warn(
                     "New index in an already populated table. %s.%s will not be indexed. "
-                    'Consider calling %s.index_table_rebuild(["%s"]) to initialize the indexation table.'
+                    'Consider calling %s.index_table_rebuild("%s") to initialize the indexation table.'
                     % (
                         self.__class__.__name__,
                         index.key,
